# HPROC - Run image processing tasks one amp at a time
# January 25, 2005
# By Doug Mink, SAO Telescope Data Center

procedure procamps (images)

string	images=""	{prompt="Images to merge"}

bool	fixpreamp=yes	{prompt="Run preampfix to clean overscan (y or n)"}
string	biassec="[1080:1120,3:4610]"	{prompt="Bias section"}
bool	fixpix=yes	{prompt="Fix bad pixels (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, temp1, temp2, temp3, im2, im4, imin, templist, imu
string	chip1, chip2, arg1, imx, ims, imsec, history, tchar
bool	dellist, vb, fixpre

	vb = verbose
	imx = ".fits"
	ims = image
	fixpre = fixpreamp

	if (ims == "") {
	    print ("HPROC: Run image processing tasks one amp at a time")
	    print ("Usage: hproc <input images with 4 extensions>")
	    return
	    }

# Make directory in which to save raw multi-extension files
	if (saveold) {
	    if (!access ("Raw"))
		mkdir ("Raw")

# Make directory in which to save preampfixed multi-extension files
	    if (!access ("Unproc"))
		mkdir ("Unproc")
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	if (tchar == '@') {
	    templist = substr (ims,2,100)
	    deltemp = no
	    }
	else {
	    templist = mktemp ("proclist")
	    files (ims, >templist)
	    }
	list = templist
        while (fscan (list,imin) != EOF) {

# Do not process images without extensions
	    imgetp.default = "-1"
	    imsec = imin // "[0]"
	    imgetp (imsec, "i_naxis")
	    if (imgetp.value != "0") {
		print ("HPROC: ", imin, " is not a multi-extension image")
		print ("NAXIS = ", imgetp.value)
		next
		}

# Do not process image twice
	    imgetp.default = "0"
	    imsec = imin // "[1]"
	    imgetp (imsec, "HPROC")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HPROC: ", imin, " has already been processed")
		    print ("HPROC = ", imgetp.value)
		    }
		next
		}

# Find root name of image
	    imroot = imin
	    smatch (imroot, imx)
	    if (smatch.found) {
		imroot = substr (imroot,1,smatch.start-1)
		}
	    else {
		imroot = imageout
		imin = imin // imx
		}
#	    if (vb) {
#		print ("HPROC: image root is ",imroot)
#		}

# Fix preamp output using Brian McLeod's task
	    if (fixpre) {
		temp1 = mktemp (imroot) // "p.fits"
		if (verbose)
		    print ("HPROC: Making preamp correction for ",imin)
		preampfix (imin, temp1)
		if (verbose)
		    print ""

# Make history line including time for preamp correction
        	gettime ()
		history = gettime.date // " " // gettime.time // " preamplifier correction made"
		imsec = temp1 // "[1]"
		hedit (imsec, "PREAMPFX", history, add+, verify-)
		imsec = temp1 // "[2]"
		hedit (imsec, "PREAMPFX", history, add+, verify-)
		imsec = temp1 // "[3]"
		hedit (imsec, "PREAMPFX", history, add+, verify-)
		imsec = temp1 // "[4]"
		hedit (imsec, "PREAMPFX", history, add+, verify-)

#  Save preampfixed file to Unproc/
		if (saveold) {
		    imu = "Unproc/" // imin
		    if (access (imu))
			delete (imu)
		    copy (temp1, imu)
		    }
		}
	    else {
		temp1 = imin
		}

#  Trim, subtract overscan, and fix bad pixels using mscred.ccdproc
	    ccdproc.bpmasks = ""
	    ccdproc.ccdtype = ""
	    ccdproc.noproc = no
	    ccdproc.xtalkcor = no
	    ccdproc.fixpix = fixpix
	    ccdproc.overscan = no
	    ccdproc.trim = no
	    ccdproc.zerocor = no
	    ccdproc.darkcor = no
	    ccdproc.flatcor = no
	    ccdproc.sflatcor = no
	    ccdproc.split = no
	    ccdproc.merge = no
	    ccdproc.xtalkfile = ""
	    ccdproc.fixfile = "BPM"
	    ccdproc.saturation = "INDEF"
	    ccdproc.sgrow = 0
	    ccdproc.bleed = "INDEF"
	    ccdproc.btrail = 20
	    ccdproc.bgrow = 0
	    ccdproc.biassec = biassec
	    ccdproc.trimsec = "!TRIMSEC"
	    ccdproc.zero = "ZERO"
	    ccdproc.dark = "DARK"
	    ccdproc.flat = "FLAT"
	    ccdproc.sflat = ""
	    ccdproc.minreplace = 1.
	    ccdproc.interactive = no
	    ccdproc.function = "legendre"
	    ccdproc.order = 1
	    ccdproc.sample = "*"
	    ccdproc.naverage = 1
	    ccdproc.niterate = 1
	    ccdproc.low_reject = 3.
	    ccdproc.high_reject = 3.
	    ccdproc.grow = 0.
	    ccdproc.fd = ""
	    ccdproc.fd2 = ""
	    temp2 = mktemp (imroot) // "pc" // imx
	    if (verbose)
		print ("HPROC: fixpix, overscan, and trim ", imin)
	    mscred.ccdproc (temp1, output=temp2, overscan+, trim+)

#  Adjust amplifiers to same gain
	    temp3 = mktemp (imroot) // "pcg" // imx
	    if (verbose)
		print ("HPROC: gain correction for ", imin)
	    gaincorr (temp2, temp3)

#  Rename original file
	    if (saveold) {
		imu = "Raw/" // imin
		if (access (imu))
		    delete (imu)
		rename (imin, imu)
		}
	    else {
		delete (imin)
		}

#  Rename result to same as original file
	    rename (temp3, imin)
	    if (verbose)
		print ("HPROC: ",temp3," -> ", imin)

	    if (deltemp) {
		if (fixpre)
		    imdelete (temp1)
		imdelete (temp2)
		if (verbose)
		    print ("HPROC: Deleting ",temp1," and ",temp2)
		}

# Add history line including time
	    gettime ()
	    history = gettime.date // " " // gettime.time // " fixpix, overscan, and trim"
	    imsec = imin // "[1]"
            hedit (imsec, "HPROC", history, add+, verify-)
	    imsec = imin // "[2]"
            hedit (imsec, "HPROC", history, add+, verify-)
	    imsec = imin // "[3]"
            hedit (imsec, "HPROC", history, add+, verify-)
	    imsec = imin // "[4]"
            hedit (imsec, "HPROC", history, add+, verify-)
	    }
	if (deltemp)
	    delete (templist)
end

# May 20 2004	New program to process list of multiextension images
# May 21 2004	Add header line for preamp fix; move original file to Raw/
# Jun  8 2004	Change name of task from PROCAMPS to HPROC
# Jun  8 2004	Save preampfixed, unccdproc'ed files to Unproc/
# Jun 23 2004	Save unprocessed files only if saveold=yes
# Sep  2 2004	Add HPROC keyword and skip file if it exists already
# Oct 13 2004	Write HPROC keyword to each amplifier's image extension
# Oct 18 2004	Print HPROC keyword if it is present
# Dec 17 2004	Skip images where NAXIS in primary header is non-zero
# Dec 20 2004	Skip preampfix if fixpreamp parameter is no
#
# Jan 25 2005	Add fixpix parameter to set ccdproc.fixpix
