# HSPLIT - Split multispec hectospec file into 1-D spectra
# By Doug Mink, SAO Telescope Data Center
# April 11, 2005

procedure hsplit (msfile)

string	msfile=""	{prompt="Aperture reference multispec file (drop .ms)"}
file	keywords="hectospec$lib/keywords" {prompt="File containing header keywords to be deleted"}
bool	deltemp=yes	{prompt="Delete temporary list file (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

	file	keyfile, keypath, imin, imroot, templist
	int	lroot
	string	arg, tchar, ims, history
	bool	vb

	vb = verbose
	ims = msfile
	keyfile = keywords

	tchar = substr (ims,1,1)
	if (tchar == '@') {
	    templist = substr (ims,2,100)
	    deltemp = no
	    }
	else {
	    templist = mktemp ("splitlist")
	    files (ims, >templist)
	    }
	list = templist
	while (fscan (list,imin) != EOF) {

# Make sure that input is a multispec file
	    smatch (imin, ".ms")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		}
            else {
		smatch (imin, ".fits"
		if (smatch.found) {
		    lroot = smatch.start - 1
		    imroot = substr (imin, 1, lroot)
		    imnew = imroot // ".ms.fits"
		    if (access (imnew)) {
			imin = imnew
			}
		    else {
			print ("HSPLIT: ", imin," is not a multispec file")
			next
			}
		    }
		else {
		    imroot = imin
		    imin = imroot // ".ms.fits"
		    if (!access (imin)) {
			print ("HSPLIT: ", imin," is not a multispec file")
			next
			}
		    }
        	}


# Do not split an image twice
	    imgetp.default = "0"
	    imgetp (imin, "HSPLIT")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HSPLIT: ", imin, " has already been split")
		    print ("HSPLIT = ", imgetp.value)
		    }
		next
		}

# Set list of keywords to delete from output files
	    if (keyfile == "") {
		keypath = ""
		}
	    else {
		keypath = osfn (keywords)
		}

# Set up arguments and run hsplit shell script
	    if (keypath == "") {
		arg = ""
		}
	    else {
		arg = "-k " // keypath
		}
	    if (vb) {
		arg = arg // " -v "
		}
	    arg = arg // " " // imroot
	    hsplitsh (arg)

# Add history line including time
	    gettime ()
	    history = gettime.date // " " // gettime.time // " image split to " // "1d." // imroot
	    if (vb) {
		hedit (imin, "HSPLIT", history, add+, verify-, show+)
		}
	    else {
		hedit (imin, "HSPLIT", history, add+, verify-, show-)
		}
	    }

end

# Jan  8 2004	New script to run shell script
# Jun  9 2004	Deal with various forms of input filename
# Sep  2 2004	Split multiple files from list or list file

# Jan 11 2005	Improve input filename handling
# Apr 11 2005	Check HSPLIT keyword before splitting; set afterward
# Apr 11 2005	Change temporary file name to splitlist*
