# XCSKY - Piecewise cross-correlate a sky spectrum against a template sky
#	  spectrum with a wavelength solution in wavelength space using
#	  MKXCTEMP and XCSAO, and make a new sky file id file
# June 25, 2003
# By Doug Mink, SAO Telescope Data Center

procedure xcsky (object)

file	object		{prompt="Object spectrum file"}

int	aperture=0	{prompt="Object spectrum aperture"}
file	template	{prompt="Template spectrum file"}
file	ttemplate	{prompt="Shifted and stretched template file"}
int	firstwav=4000	{prompt="First wavelength in angstroms to correlate"}
int	lastwav=8000	{prompt="Last wavelength in angstroms to correlate"}
int	nwav=200	{prompt="Number of angstroms to correlate"}
int	nbin=256	{prompt="Number of rebinned pixels to correlate"}
int	intwav=100	{prompt="Interval between correlation centers in angstroms"}
real	ndev=0		{prompt="Reject past this number of standard deviations"}
bool	interactive=no	{prompt="Fit dispersion function interactively (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

begin
	char arg
	int	objap
	file templist	# File to which to write list of segments
	file xcout	# File to which to write xcsao output
	file objfile, tempfile, ttempfile, obj
	file dbfile	# id file reated by reref
	int fw, lw, dw, flw

	real constant
	real nd
	int im
	bool intact

	fw = firstwav
	lw = lastwav
	dw = intwav

	objap = aperture
	if (ndev < 1) {
	    nd = 10
	    }
	else {
	    nd = ndev
	    }
	intact = interactive
	objfile = object
	tempfile = template
	ttempfile = ttemplate

# If template file has negative values, put a pedestal under it
	saotdc
	itools
	procdata
	istat (tempfile)
	if (istat.min < 0.0) {
	    constant = -istat.min + 100.0
	    imarith (tempfile, "+", constant, tempfile, verbose=debug)
	    }

# If object file has negative values, put a pedestal under it
	if (objap > 0) {
	    obj = objfile // "[*," // objap // "]"
	    }
	else {
	    obj = objfile
	    }
	istat (obj)
	if (istat.min < 0.0) {
	    constant = -istat.min + 100.0
	    imarith (objfile, "+", constant, objfile, verbose=debug)
	    }

# Make list of templates nwav wide every intwav pixels
	limwav (objfile)
	if (fw == INDEF || limwav.lambda1 > fw) {
	    fw = limwav.lambda1
	    }
	flw = limwav.lambda2 - nwav
	if (lw == INDEF || flw < lw) {
	    lw = limwav.lambda2
	    }
	templist = mktemp ("xcsky")
	mkxctempw (tempfile,templist=templist,firstwav=fw,lastwav=lw,nwav=nwav,intwav=dw)

# Cross-correlate spectrum against template, producing output in xcout
	rvsao (motd=no)
	arg = "@" // templist
	xcout = mktemp ("xcsao")
	if (objap > 0) {
	    obj = objfile // "[" // objap // "]"
	    }
	else {
	    obj = objfile
	    }
	xcsao (objfile,templates=arg,correlate="wavelength",specnum=0,tempnum=0,ncols=nbin,report_mode=16,logfile=xcout)
	if (debug) {
	    print ("xcsao output written to ",xcout)
	    }
	delete (templist)

# If spectrum file does not exist for shifted template, create it
	if (!access (ttempfile)) {
	    printf ("Fake template file %s does not exist\n", ttempfile)
	    imcopy (tempfile, ttempfile)
	    }

# Read pixels and shifts from xcout and fit a curve to them, then make
# a new database/id file with modified pixel positions for lines
	dbfile = "database//id" // ttempfile
	delete (dbfile)
	
	skycal (tempfile, refout=ttempfile, shiftfile=xcout, verbose=verbose, debug=debug)

	delete (xcout)
end

# May 21 2003	New task based on reref
# Jun 18 2003	Use mkxctempw for calibrated spectra
